@@ -877,7 +877,7 @@ static int APR_THREAD_FUNC regfnServerSupportFunction(isapi_cid    *cid,
         /* Set the status to be returned when the HttpExtensionProc()
          * is done.
          * WARNING: Microsoft now advertises HSE_REQ_SEND_URL_REDIRECT_RESP
-         *          and HSE_REQ_SEND_URL as equivalant per the Jan 2000 SDK.
+         *          and HSE_REQ_SEND_URL as equivalent per the Jan 2000 SDK.
          *          They most definitely are not, even in their own samples.
          */
         apr_table_set (r->headers_out, "Location", buf_data);
@@ -894,7 +894,7 @@ static int APR_THREAD_FUNC regfnServerSupportFunction(isapi_cid    *cid,
         }
 
         /* Reset the method to GET */
-        r->method = apr_pstrdup(r->pool, "GET");
+        r->method = "GET";
         r->method_number = M_GET;
 
         /* Don't let anyone think there's still data */
