@@ -180,7 +180,7 @@ static void chdir_for_gprof(void)
         }
     }
     else {
-        use_dir = ap_server_root_relative(pconf, DEFAULT_REL_RUNTIMEDIR);
+        use_dir = ap_runtime_dir_relative(pconf, "");
     }
 
     chdir(use_dir);
@@ -564,9 +564,16 @@ static void child_main(int child_num_arg)
 
         status = apr_pollset_add(pollset, &pfd);
         if (status != APR_SUCCESS) {
-            ap_log_error(APLOG_MARK, APLOG_EMERG, status, ap_server_conf, APLOGNO(00157)
-                         "Couldn't add listener to pollset; check system or user limits");
-            clean_child_exit(APEXIT_CHILDSICK);
+            /* If the child processed a SIGWINCH before setting up the
+             * pollset, this error path is expected and harmless,
+             * since the listener fd was already closed; so don't
+             * pollute the logs in that case. */
+            if (!die_now) {
+                ap_log_error(APLOG_MARK, APLOG_EMERG, status, ap_server_conf, APLOGNO(00157)
+                             "Couldn't add listener to pollset; check system or user limits");
+                clean_child_exit(APEXIT_CHILDSICK);
+            }
+            clean_child_exit(0);
         }
 
         lr->accept_func = ap_unixd_accept;
@@ -641,7 +648,7 @@ static void child_main(int child_num_arg)
                     ap_log_error(APLOG_MARK, APLOG_ERR, status,
                                  ap_server_conf, APLOGNO(00158) "apr_pollset_poll: (listen)");
                     SAFE_ACCEPT(accept_mutex_off());
-                    clean_child_exit(1);
+                    clean_child_exit(APEXIT_CHILDSICK);
                 }
 
                 /* We can always use pdesc[0], but sockets at position N
@@ -678,7 +685,7 @@ static void child_main(int child_num_arg)
 
         if (status == APR_EGENERAL) {
             /* resource shortage or should-not-occur occured */
-            clean_child_exit(1);
+            clean_child_exit(APEXIT_CHILDSICK);
         }
         else if (status != APR_SUCCESS) {
             continue;
